"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormController = void 0;
const Validators_1 = require("./Validators");
const Guards_1 = require("./Guards");
const ValueStorage_1 = require("./Models/ValueStorage");
const Errors_1 = require("./Models/Errors");
class FormController {
    constructor(config) {
        this.config = config;
        this.guards = new Map();
        this.validators = new Map();
        this.errors = new Errors_1.Errors();
        this.values = new ValueStorage_1.ValueStorage(config.fields);
        for (let key in config.fields) {
            const cur = config.fields[key];
            if (config.guards && config.guards[key]) {
                const def = Guards_1.getDefaultGuard(cur);
                if (def) {
                    this.guards.set(key, Guards_1.combineGuards(def, config.guards[key]));
                }
                else {
                    this.guards.set(key, config.guards[key]);
                }
            }
            else {
                const def = Guards_1.getDefaultGuard(cur);
                if (def)
                    this.guards.set(key, def);
            }
            if (config.fields[key].required) {
                if (config.validators && config.validators[key]) {
                    this.validators.set(key, Validators_1.combineValidators(Validators_1.RequiredValidator, config.validators[key]));
                }
                else {
                    this.validators.set(key, Validators_1.RequiredValidator);
                }
            }
            else {
                if (config.validators && config.validators[key]) {
                    this.validators.set(key, config.validators[key]);
                }
            }
        }
    }
    set(name, value) {
        this.change(name, value);
    }
    get(name) {
        return this.values.get(name);
    }
    change(name, nextValue) {
        if (!(name in this.config.fields))
            return;
        let value = nextValue;
        const guard = this.guards.get(name);
        if (guard) {
            const guardResult = guard(value);
            if (guardResult instanceof Error) {
                return console.error(guardResult.message);
            }
            value = guardResult;
        }
        const validator = this.validators.get(name);
        if (validator) {
            const result = validator(nextValue, this.values.getState());
            if (result instanceof Error) {
                this.errors.set(name, result.message);
            }
            else {
                this.errors.remove(name);
            }
        }
        this.values.set(name, value);
    }
    getValues() {
        return this.values.getState();
    }
    submit(...arg) {
        if (this.errors.size() > 0)
            return this.errors.getState();
        const state = this.values.getState();
        const errors = {};
        for (let key in this.config.fields) {
            const validator = this.validators.get(key);
            if (!validator)
                continue;
            const result = validator(state[key], state);
            if (result instanceof Error) {
                errors[key] = result.message;
            }
        }
        if (Object.keys(errors).length > 0) {
            this.errors.setState(errors);
            return this.errors.getState();
        }
        const sub = this.config.submit(state, ...arg);
        if (!sub)
            return;
        if (sub instanceof Promise) {
            sub
                .then(res => {
                if (!res)
                    return;
                this.errors.setState(res);
            })
                .catch(console.error);
        }
        else {
            this.errors.setState(sub);
        }
    }
    subscribe(type, cb) {
        if (type === "errors")
            return this.errors.subscribe(cb);
        else
            return this.values.subscribe(cb);
    }
}
exports.FormController = FormController;
//# sourceMappingURL=FormController.js.map